// FIXME: These properties also exist in app-utils.
const SET_COOKIE_MESSAGE = 'set_cookie'
const GET_COOKIE_MESSAGE = 'get_cookie'
const MAX_RESULTS = 25000

const POST_INSTALL_PORT = 'post_install'
const UNINSTALL_URL_PORT = 'uninstall_url'
const GOOGLE_CONTACTS_PORT = 'google_contacts'
const AUTOCOMPLETE_CONTACTS_PORT = 'autocomplete_contacts'
const HEARTBEAT_PORT = 'heartbeat'
const BADGE_PORT = 'set_badge'

var apiHost = "https://www.amitree.com"

class BackgroundPage {
  constructor() {
    this.installDetails = null
    this.onboardingShown = false

    this.onInstallBackgroundListener = this.onInstallBackgroundListener.bind(this)
    this.onInstallFacebookListener = this.onInstallFacebookListener.bind(this)
    this.onInstallGoogleListener = this.onInstallGoogleListener.bind(this)
    this.setMailChimpConfigurationForTracking = this.setMailChimpConfigurationForTracking.bind(this)
    this.onConnectListener = this.onConnectListener.bind(this)
    this.onMessageListener = this.onMessageListener.bind(this)
    chrome.browserAction.onClicked.addListener(this.onFolioIconClicked)
    chrome.runtime.onInstalled.addListener(this.onInstallBackgroundListener)
    chrome.runtime.onInstalled.addListener(this.onInstallFacebookListener)
    chrome.runtime.onInstalled.addListener(this.onInstallGoogleListener)
    chrome.runtime.onInstalled.addListener(this.setMailChimpConfigurationForTracking)
    chrome.runtime.onInstalled.addListener(function(details) {
      if (details.reason === "install") {
        chrome.tabs.create({
          url: `${apiHost}/folio/install_redirect`
        }, function(_tab) {})
      }
    })
    chrome.runtime.onConnect.addListener(this.onConnectListener)
    chrome.runtime.onMessage.addListener(this.onMessageListener)
    chrome.runtime.onMessageExternal.addListener(this.extensionInstalled)
    if("" !== "") {
      chrome.browserAction.setBadgeText({text: ""})
    }
    chrome.runtime.setUninstallURL(`${apiHost}/folio/uninstall`)
  }

  onInstallBackgroundListener(details) {
    this.installDetails = details
  }

  onInstallFacebookListener(details) {
    if (details.reason !== 'install') {
      return
    }

    $('head').append(
      `
        <!-- Facebook Conversion Code for Agent installs Folio -->
        <img height="1" width="1" alt="" style="display:none" src="https://www.facebook.com/tr?ev=6039464098397&amp;cd[value]=0.00&amp;cd[currency]=USD&amp;noscript=1" />\
      `
    )
  }

  onInstallGoogleListener(details) {
    if (details.reason !== 'install') {
      return
    }

    const $pixel = $('<img height="1" width="1" style="border-style:none;" alt="" src="https://www.googleadservices.com/pagead/conversion/999678629/?label=i2u0CKnkqmkQpcXX3AM&amp;guid=ON&amp;script=0"/>')
    $('body').append($pixel)
  }

  setMailChimpConfigurationForTracking(details) {
    if (details.reason !== 'install') {
      return
    }

    window.$mcGoal = {'settings': {'uuid': 'a7afe917d7166b44ba07b576d', 'dc': 'us8'}}
  }

  onConnectListener(port) {
    port.onMessage.addListener(message => {
      switch (port.name) {
        case POST_INSTALL_PORT:
          if ((message === 'ready') && !this.onboardingShown && Ember.isPresent(this.installDetails)) {
            port.postMessage(this.installDetails)
          } else if (message === 'onboarding-shown') {
            this.onboardingShown = true
          }
          break

        case GOOGLE_CONTACTS_PORT:
          this.requestGoogleContacts(message, response => port.postMessage(response))
          break

        case AUTOCOMPLETE_CONTACTS_PORT:
          this.downloadAutocompleteContacts(message, response => port.postMessage(response))
          break

        case UNINSTALL_URL_PORT:
          apiHost = message.apiHost
          chrome.runtime.setUninstallURL(`${message.apiHost}/folio/uninstall?install_id=${message.uniqueId}`)
          break

        case BADGE_PORT:
          chrome.browserAction.setBadgeText({text: message.text})
          chrome.browserAction.setBadgeBackgroundColor({color: message.color})
          break
      }
    })
  }

  onFolioIconClicked() {
    const folioPageUrl = `${apiHost}/folio?utm_medium=chrome&utm_source=folio_button`

    chrome.tabs.query({title: 'Folio'}, tabs => {
      if (tabs.length) {
        const folioTabId = tabs[0].id
        chrome.tabs.update(folioTabId, {url: folioPageUrl, active: true})
      } else {
        chrome.tabs.create({url: folioPageUrl})
      }
    })
  }

  downloadAllGoogleContacts(token, callback, maxResults = MAX_RESULTS, startIndex = 1) {
    const params = $.param({
      alt: 'json',
      access_token: token,
      'max-results': maxResults,
      'start-index': startIndex,
      v: '3.0'
    })

    $.get(`https://www.google.com/m8/feeds/contacts/default/thin?${params}`, (response) => {
      callback(response)

      const { feed } = response
      const returnedOnThisPage = feed.entry.length

      if (!returnedOnThisPage) {
        return
      }

      const total = +feed.openSearch$totalResults.$t
      const fromIndex = +feed.openSearch$startIndex.$t
      const perPage = +feed.openSearch$itemsPerPage.$t
      const alreadyReturned = fromIndex + perPage - 1

      if (total > alreadyReturned && maxResults > perPage) {
        this.downloadAllGoogleContacts(token, callback, maxResults - returnedOnThisPage, fromIndex + perPage)
      }
    })
  }

  requestGoogleContacts(accessToken, callback) {
    this.downloadAllGoogleContacts(accessToken, callback, MAX_RESULTS)
  }

  downloadAutocompleteContacts(endpointUrl, callback) {
    $.get(endpointUrl, callback)
  }

  onMessageListener(message, _sender, sendResponse) {
    if (message.name === SET_COOKIE_MESSAGE) {
      chrome.cookies.set.apply(chrome.cookies, message.args)
    } else if (message.name === GET_COOKIE_MESSAGE) {
      chrome.cookies.get(message.args[0], sendResponse)
      return typeof sendResponse === 'function' // true if sendReponse is going to be called later
    }
  }

  extensionInstalled(request, _sender, sendResponse) {
    if (request && request.message === 'installed') {
      sendResponse({installed: true})
    }

    return true // keeps the listener open
  }
}

// The background page instance is persisted. Reloading the browser will not create a new one.
window.BackgroundPage = BackgroundPage
